<?php

/**
 * Builds the form for an overview of the newsletters.
 */
function sunmailer_content_overview_newsletters() {
  $newsletters = sunmailer_newsletter_load_all(TRUE);
  $base = 'admin/settings/sunmailer/content';
  $list = array();
  $form = array();
  $form['content']['#tree'] = TRUE;
  $newsletters_delta = count($newsletters);
  foreach ($newsletters as $nlid => $newsletter) {
    $list[$nlid] = array();
    $form['content']['newsletter'][$nlid] = array(
      'title' => array(
        '#type' => 'markup',
        '#value' => $newsletter->title,
      ),
      'edit' => array(
        '#type' => 'markup',
        '#value' => l('edit', "$base/newsletter/$nlid/edit"),
      ),
      'delete' => array(
        '#type' => 'markup',
        '#value' => l('delete', "$base/newsletter/$nlid/delete"),
      ),
      'add_section' => array(
        '#type' => 'markup',
        '#value' => l('add section', "$base/section/add/$nlid"),
      ),
      'weight' => array(
        '#type' => 'weight',
        '#delta' => $newsletters_delta,
        '#default_value' => $newsletter->weight,
      ),
    );
    $sections_delta = count($newsletter->sections);
    foreach ($newsletter->sections as $sid => $section) {
      $list[$nlid][] = $sid;
      $form['content']['section'][$sid] = array(
        'title' => array(
          '#type' => 'markup',
          '#value' => $section->title,
        ),
        'edit' => array(
          '#type' => 'markup',
          '#value' => l('edit', "$base/section/$sid/edit"),
        ),
        'delete' => array(
          '#type' => 'markup',
          '#value' => l('delete', "$base/section/$sid/delete"),
        ),
        'weight' => array(
          '#type' => 'weight',
          '#delta' => $sections_delta,
          '#default_value' => $section->weight,
        ),
      );
    }
  }
  $form['list'] = array(
    '#type' => 'value',
    '#value' => $list,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );
  return $form;
}

/**
 * Submission callback for the newsletters overview form.
 */
function sunmailer_content_overview_newsletters_submit($form, &$form_state) {
  $values = $form_state['values'];
  foreach ($values['list'] as $nlid => $sids) {
    db_query("UPDATE {sunmailer_newsletters} SET weight = %d WHERE nlid = %d", $values['content']['newsletter'][$nlid]['weight'], $nlid);
    foreach ($sids as $sid) {
      db_query("UPDATE {sunmailer_sections} SET weight = %d WHERE sid = %d", $values['content']['section'][$sid]['weight'], $sid);
    }
  }
}

/**
 * Callback to generate a date in the specified format.
 */
function sunmailer_date_format() {
  print drupal_to_js(array('status' => TRUE, 'data' => t(date($_POST['date_format']))));
  exit;
}

/**
 * Builds the form for a newsletter.
 */
function sunmailer_content_newsletter_form($form_state = NULL, $newsletter = NULL) {
  // Generate the design options.
  $design_options = array();
  $alt_design_options = array(0 => t('(None)'));
  foreach (sunmailer_design_load_all() as $did => $design) {
    $design_options[$did] = $design->title;
    if (!$design->html) {
      $alt_design_options[$did] = $design->title;
    }
  }
  // Build the form.
  $form = array();
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter ? $newsletter->nlid : NULL,
  );
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#description' => t('Subject line for the email'),
    '#required' => TRUE,
    '#default_value' => $newsletter ? $newsletter->subject : '',
  );
  $form['design'] = array(
    '#type' => 'select',
    '#title' => t('Design'),
    '#options' => $design_options,
    '#required' => TRUE,
    '#default_value' => $newsletter ? $newsletter->did : 1,
  );
  $form['alt_design'] = array(
    '#type' => 'select',
    '#title' => t('Alternate design'),
    '#description' => t('(Optional) Text design for email clients which do not support HTML'),
    '#options' => $alt_design_options,
    '#required' => TRUE,
    '#default_value' => $newsletter ? $newsletter->alt_did : 0,
  );
  $form['header'] = array(
    '#type' => 'fieldset',
    '#title' => t('Header'),
    '#collapsible' => FALSE,
  );
  $form['header']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#description' => t('Leave blank to use the site name'),
    '#required' => TRUE,
    '#default_value' => $newsletter ? $newsletter->title : '',
  );
  $form['header']['logo_fid'] = array(
    '#type' => 'value',
    '#default_value' => $newsletter ? $newsletter->logo_fid : 0,
  );
  $form['header']['logo'] = array(
    '#type' => 'file',
    '#title' => t('Logo'),
    '#description' => t('(Optional) Logo image, with the title as the alternate text'),
  );
  if ($newsletter && $newsletter->logo_fid) {
    $filename = db_result(db_query("SELECT filename FROM files WHERE fid = %d", $newsletter->logo_fid));
    $form['header']['filename'] = array(
      '#type' => 'markup',
      '#prefix' => '<p>',
      '#value' => t('Current logo: !filename', array('!filename' => $filename)),
      '#suffix' => '</p>',
    );
    $form['header']['delete_logo'] = array(
      '#type' => 'checkbox',
      '#title' => t('Delete logo'),
    );
  }
  $form['header']['date_format'] = array(
    '#type' => 'textfield',
    '#title' => t('Date format'),
    '#description' => t('(Optional) PHP <a href="http://www.php.net/manual/en/function.date.php">date format</a> for the date in the header'),
    '#ahah' => array(
      'path' => 'sunmailer/date_format',
      'wrapper' => 'display-date-format',
    ),
    '#default_value' => $newsletter ? $newsletter->date_format : 'l, j F, Y'
  );
  $form['header']['display_date_format'] = array(
    '#type' => 'markup',
    '#prefix' => '<div id="display-date-format">',
    '#value' => t(date($form['header']['date_format']['#default_value'])) . ' ',
    '#suffix' => '</div>'
  );
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['advanced']['empty'] = array(
    '#type' => 'checkbox',
    '#title' => t('Display empty section'),
    '#description' => t('If the section\'s view has empty text, this text will be displayed'),
    '#default_value' => $newsletter ? $newsletter->empty : '',
  );
   $form['advanced']['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#description' => t('Orders newsletters on the subscription form'),
    '#default_value' => $newsletter ? $newsletter->weight : 0,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validation callback for the newsletter form.
 */
function sunmailer_content_newsletter_form_validate($form, &$form_state) {
  if (!$form_state['values']['delete_logo']) {
    if ($file = file_save_upload('logo', array('file_validate_is_image' => array()), file_directory_path())) {
      $form_state['values']['logo_file'] = $file;
    }
  }
}

/**
 * Submission callback for the newsletter form.
 */
function sunmailer_content_newsletter_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $newsletter = new stdClass();
  $newsletter->nlid = $values['nlid'];
  $newsletter->subject = $values['subject'];
  $newsletter->did = $values['design'];
  $newsletter->alt_did = $values['alt_design'];
  $newsletter->title = $values['title'];
  if ($values['delete_logo']) {
    _sunmailer_file_delete($values['logo_fid']);
    $newsletter->logo_fid = 0;
  }
  elseif ($file = $values['logo_file']) {
    file_set_status($file, FILE_STATUS_PERMANENT);
    $newsletter->logo_fid = $file->fid;
    if ($values['logo_fid']) {
      _sunmailer_file_delete($values['logo_fid']);
    }
  }
  else {
    $newsletter->logo_fid = $values['logo_fid'];
  }
  $newsletter->date_format = $values['date_format'];
  $newsletter->empty = (bool)$values['empty'];
  $newsletter->weight = $values['weight'];
  drupal_write_record('sunmailer_newsletters', $newsletter, $newsletter->nlid ? 'nlid' : array());
  if ($values['nlid']) {
    drupal_set_message('Newsletter successfully edited');
    $form_state['redirect'] = 'admin/settings/sunmailer/content';
  }
  else {
    drupal_set_message('Newsletter successfully added');
  }
  menu_cache_clear_all();
}

/**
 * Confirmation form to delete a newsletter.
 */
function sunmailer_content_newsletter_delete_confirm($form_state = NULL, $newsletter) {
  $question = t('Are you sure you want to delete the newsletter %title?', array('%title' => $newsletter->title));
  $description = t('This will also delete associated sections and subscriptions.');
  $form = array();
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $newsletter->nlid,
  );
  return confirm_form($form, $question, 'admin/settings/sunmailer/content', $description, t('Delete'));
}

/**
 * Submission callback for newsletter deletion form.
 */
function sunmailer_content_newsletter_delete_confirm_submit($form, &$form_state) {
  sunmailer_newsletter_delete($form_state['values']['nlid']);
  drupal_set_message(t('Newsletter successfully deleted'));
  $form_state['redirect'] = 'admin/settings/sunmailer/content';
  menu_cache_clear_all();
}


/**
 * Autocomplete callback for a view and an associated display.
 */
function sunmailer_autocomplete_views($view_string = '', $display_string = '') {
  $views = views_get_all_views();
  $matches = array();
  foreach ($views as $view) {
    if ($view->disabled) {
      continue;
    }
    if ($view->base_table != 'node') {
      continue;
    }
    foreach ($view->display as $did => $display) {
      $match = "{$view->name}/$did";
      $string = $view_string . ($display_string ? "/$display_string" : '');
      $match_display = "$match ({$display->display_title})";
      if (strpos($match, $string) === 0) {
        $matches[$match] = $match_display;
      }
    }
  }
  drupal_json(array_slice($matches, 0, 10));
}

/**
 * Builds the form for a newsletter section.
 */
function sunmailer_content_section_form($form_state = NULL, $section = NULL, $newsletter = NULL) {
  $form = array();
  $form['sid'] = array(
    '#type' => 'value',
    '#value' => $section ? $section->sid : NULL,
  );
  $form['nlid'] = array(
    '#type' => 'value',
    '#value' => $section ? $section->nlid : $newsletter->nlid,
  );
  $form['section'] = array(
    '#type' => 'fieldset',
    '#title' => 'Section',
    '#collapsible' => FALSE,
  );
  $form['section']['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#required' => TRUE,
    '#default_value' => $section ? $section->title : '',
  );
  $form['section']['view_display'] = array(
    '#type' => 'textfield',
    '#title' => t('View'),
    '#description' => t('Format is "view/display" (e.g.: archive/default)'),
    '#required' => TRUE,
    '#default_value' => $section ? "{$section->view}/{$section->display}" : '',
    '#autocomplete_path' => 'sunmailer/autocomplete/views',
  );
  $form['section']['args'] = array(
    '#type' => 'textfield',
    '#title' => t('Arguments'),
    '#description' => t('Separate arguments with a / as though they were a URL path'),
    '#default_value' => $section ? $section->args : '',
  );
  if (!$section) {
    $form['subscribe'] = array(
      '#type' => 'radios',
      '#title' => 'Subscriptions',
      '#options' => array(
        'any' => 'Add any subscriber to this newsletter',
        'all' => 'Add only subscribers to this newsletter subscribed to all sections',
        'none' => 'Add no subscribers',
      ),
      '#default_value' => 'any',
    );
  }
  $form['advanced'] = array(
    '#type' => 'fieldset',
    '#title' => t('Advanced settings'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  $form['advanced']['weight'] = array(
    '#type' => 'weight',
    '#title' => t('Weight'),
    '#default_value' => $section ? $section->weight : 0,
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validation callback for the section form.
 */
function sunmailer_content_section_form_validate($form, &$form_state) {
  $view_display = explode('/', $form_state['values']['view_display']);
  $view = views_get_view($view_display[0]);
  if (!$view) {
    form_set_error('view_display', t('View does not exist.'));
    return;
  }
  if ($view->base_table != 'node') {
    form_set_error('view_display', t('View is not a node view.'));
    return;
  }
  if ($view->disabled) {
    form_set_error('view_display', t('View is disabled.'));
    return;
  }
  if (!array_key_exists($view_display[1], $view->display)) {
    form_set_error('view_display', t('View does not contain this display.'));
    return;
  }
}

/**
 * Submission callback for the section form.
 */
function sunmailer_content_section_form_submit($form, &$form_state) {
  // Save the section.
  $values = $form_state['values'];
  $section = new stdClass();
  $section->sid = $values['sid'];
  $section->nlid = $values['nlid'];
  $section->title = $values['title'];
  $view_display = explode('/', $values['view_display']);
  $section->view = $view_display[0];
  $section->display = $view_display[1];
  $section->args = $values['args'];
  $section->weight = $values['weight'];
  drupal_write_record('sunmailer_sections', $section, $section->sid ? 'sid' : array());
  if ($values['sid']) {
    drupal_set_message(t('Section successfully edited.'));
    $form_state['redirect'] = 'admin/settings/sunmailer/content';
  }
  else {
    drupal_set_message(t('Section successfully added.'));
  }
  // Add new subscriptions.
  switch ($values['subscribe']) {
  case 'any':
    db_query("INSERT INTO {sunmailer_subscriptions} (SELECT DISTINCT(su.smid), %d FROM {sunmailer_sections} se, {sunmailer_subscriptions} su WHERE se.sid = su.sid AND se.nlid = %d)", $section->sid, $section->nlid);
    break;
  case 'all':
    $newsletter = sunmailer_newsletter_load($section->nlid);
    $count = db_result(db_query("SELECT COUNT(*) FROM {sunmailer_sections} WHERE nlid = %d", $section->nlid)) - 1;
    db_query("INSERT INTO {sunmailer_subscriptions} (SELECT su.smid, %d FROM {sunmailer_sections} se, {sunmailer_subscriptions} su WHERE se.sid = su.sid AND se.nlid = %d GROUP BY su.smid HAVING COUNT(su.sid) = %d)", $section->sid, $section->nlid, $count);
    break;
  case 'none':
    break;
  }
}

/**
 * Confirmation form to delete a newsletter section.
 */
function sunmailer_content_section_delete_confirm($form_state = NULL, $section) {
  $question = t('Are you sure you want to delete the section %title?', array('%title' => $section->title));
  $description = t('This will also delete associated subscriptions');
  $form = array();
  $form['sid'] = array(
    '#type' => 'value',
    '#value' => $section->sid,
  );
  return confirm_form($form, $question, 'admin/settings/sunmailer/content', $description, t('Delete'));
}

/**
 * Submission callback for the section deletion form.
 */
function sunmailer_content_section_delete_confirm_submit($form, &$form_state) {
  sunmailer_section_delete($form_state['values']['sid']);
  drupal_set_message(t('Section successfully deleted'));
  $form_state['redirect'] = 'admin/settings/sunmailer/content';
}

/**
 * Displays a list of designs.
 */
function sunmailer_content_overview_designs() {
  $designs = sunmailer_design_load_all();
  $content = 'admin/settings/sunmailer/content';
  $header = array(
    t('Title'),
    t('Type'),
    array('colspan' => 2, 'data' => t('Operations')),
  );
  $rows = array();
  foreach ($designs as $did => $design) {
    $rows[] = array(
      check_plain($design->title),
      $design->html ? t('HTML') : t('Text'),
      ($did > 2) ? l('edit', "$content/design/$did/edit") : '',
      ($did > 2) ? l('delete', "$content/design/$did/delete") : '',
    );
  }
  return theme('table', $header, $rows);
}

/**
 * Builds the form for a newsletter design.
 */
function sunmailer_content_design_form($form_state = NULL, $design = NULL) {
  if ($design && $design->did <= 2) {
    drupal_not_found();
    return;
  }
  $form = array();
  $form['#attributes'] = array('enctype' => 'multipart/form-data');
  $form['did'] = array(
    '#type' => 'value',
    '#value' => $design ? $design->did : NULL,
  );
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#required' => TRUE,
    '#default_value' => $design ? $design->title : '',
  );
  $form['html'] = array(
    '#type' => 'radios',
    '#title' => t('Type'),
    '#options' => array(
      TRUE => t('HTML'),
      FALSE => t('Text'),
    ),
    '#required' => TRUE,
    '#default_value' => $design ? $design->html : '',
  );
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $design ? $design->fid : 0,
  );
  $form['xsl_file'] = array(
    '#type' => 'file',
    '#title' => t('XSL file'),
  );
  if ($design) {
    $filename = db_result(db_query("SELECT filename FROM files WHERE fid = %d", $design->fid));
    $form['filename'] = array(
      '#type' => 'markup',
      '#prefix' => '<p>',
      '#value' => t('Current XSL file: !filename', array('!filename' => $filename)),
      '#suffix' => '</p>',
    );
  }
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  return $form;
}

/**
 * Validation callback for XSL file.
 */
function sunmailer_file_validate_is_xsl(&$file) {
  $doc = new DOMDocument();
  return $doc->load($file->filepath) ? array() : array(t('The file is not valid XSL.'));
}

/**
 * Validation callback for the design form.
 */
function sunmailer_content_design_form_validate($form, &$form_state) {
  if ($file = file_save_upload('xsl_file', array('sunmailer_file_validate_is_xsl' => array()), file_directory_path())) {
    $form_state['values']['file'] = $file;
  }
  elseif (!$form_state['values']['fid']) {
    form_set_error('xsl_file', t('XSL file field is required.'));
  }
}

/**
 * Submission callback for the design form.
 */
function sunmailer_content_design_form_submit($form, &$form_state) {
  $values = $form_state['values'];
  $design = new stdClass();
  $design->did = $values['did'];
  $design->title = $values['title'];
  $design->html = $values['html'];
  if ($file = $values['file']) {
    file_set_status($file, FILE_STATUS_PERMANENT);
    $design->fid = $file->fid;
    _sunmailer_file_delete($values['fid']);
  }
  else {
    $design->fid = $values['fid'];
  }
  drupal_write_record('sunmailer_designs', $design, $design->did ? 'did' : array());
  if ($values['did']) {
    drupal_set_message(t('Design successfully edited.'));
    $form_state['redirect'] = 'admin/settings/sunmailer/content/design';
  }
  else {
    drupal_set_message(t('Design successfully added.'));
  }
}

/**
 * Confirmation form to delete a design.
 */
function sunmailer_content_design_delete_confirm($form_state = NULL, $design) {
  if ($design->did <= 2) {
    drupal_not_found();
    return;
  }
  $question = t('Are you sure you want to delete the design %title?', array('%title' => $design->title));
  $form = array();
  $form['did'] = array(
    '#type' => 'value',
    '#value' => $design->did,
  );
  return confirm_form($form, $question, 'admin/settings/sunmailer/content/design', '', t('Delete'));
}

/**
 * Submission callback for the design deletion form.
 */
function sunmailer_content_design_delete_confirm_submit($form, &$form_state) {
  sunmailer_design_delete($form_state['values']['did']);
  drupal_set_message(t('Design successfully deleted.'));
  $form_state['redirect'] = 'admin/settings/sunmailer/content/design';
}

/**
 *
 */
function theme_sunmailer_content_overview_newsletters($form) {
  // Render the table for each newsletter.
  $output = '';
  $list = $form['list']['#value'];
  foreach ($list as $nlid => $sids) {
    drupal_add_tabledrag("sunmailer-newsletter-$nlid", 'order', 'sibling', "sunmailer-newsletter-$nlid-weight");
    $newsletter = $form['content']['newsletter'][$nlid];
    $header = array(
      drupal_render($newsletter['title']),
      drupal_render($newsletter['edit']),
      drupal_render($newsletter['delete']),
      drupal_render($newsletter['add_section']),
      ''
    );
    $rows = array();
    foreach ($sids as $sid) {
      $section = $form['content']['section'][$sid];
      $section['weight']['#attributes']['class'] = "sunmailer-newsletter-$nlid-weight";
      $rows[] = array(
        'class' => 'draggable',
        'data' => array(
          drupal_render($section['title']),
          drupal_render($section['edit']),
          drupal_render($section['delete']),
          '',
          drupal_render($section['weight']),
        ),
      );
    }
    $output .= theme('table', $header, $rows, array('id' => "sunmailer-newsletter-$nlid"));
  }
  // Render the newsletter ordering table.
  drupal_add_tabledrag('sunmailer-newsletter', 'order', 'sibling', 'sunmailer-newsletter-weight');
  $header = array(
    t('Newsletter order'),
    t('Weight'),
  );
  $rows = array();
  foreach ($list as $nlid => $v) {
    $newsletter = $form['content']['newsletter'][$nlid];
    $newsletter['weight']['#attributes']['class'] = "sunmailer-newsletter-weight";
    $rows[] = array(
      'class' => 'draggable',
      'data' => array(
        drupal_render($newsletter['title']),
        drupal_render($newsletter['weight']),
      ),
    );
  }
  $output .= '<hr />';
  $output .= theme('table', $header, $rows, array('id' => 'sunmailer-newsletter'));
  $output .= '<p>' . t('Newsletters will be displayed on the subscription form in the following order.') . '</p>';
  // Render the rest of the form.
  unset($form['content']);
  $output .= drupal_render($form);
  return $output;
}
