<?php

/**
 * Deletes an old file.
 */
function _sunmailer_file_delete($fid) {
  $path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $fid));
  db_query("DELETE FROM {files} WHERE fid = %d", $fid);
  file_delete($path);
}

/**
 * Sets the form element type based on which Date modules are enabled.
 */
function _sunmailer_set_date_type(&$element) {
  if (module_exists('date_api')) {
    $element['#type'] = module_exists('date_popup') ? 'date_popup' : 'date_select';
    $element['#format'] = 'Y-m-d H:i';
  }
  else {
    $element['#type'] = 'textfield';
    $element['#element_validate'] = array('_sunmailer_date_validate');
  }
}

/**
 * Validation callback for a date.
 */
function _sunmailer_date_validate($element, &$form_state) {
  if (!strtotime($element['#value'])) {
    form_error($element, t('Invalid date format.'));
  }
}

/**
 * Loads one or more newsletters using a query.
 */
function _sunmailer_newsletters_load($query, $args) {
  $newsletters = array();

  // Load basic information for each newsletter.
  $result = db_query($query, $args);
  while ($row = db_fetch_object($result)) {
    $newsletters[$row->nlid] = $row;
  }

  // Load extended information from other database tables.
  foreach ($newsletters as &$newsletter) {
    $newsletter->sections = sunmailer_section_load_newsletter($newsletter->nlid);
    $newsletter->design = sunmailer_design_load($newsletter->did);
    $newsletter->alt_design = $newsletter->alt_did ? sunmailer_design_load($newsletter->alt_did) : NULL;
  }

  return $newsletters;
}

/**
 * Loads one newsletter.
 */
function sunmailer_newsletter_load($nlid) {
  $query = "SELECT * FROM {sunmailer_newsletters} WHERE nlid = %d";
  $newsletters = _sunmailer_newsletters_load($query, $nlid);
  return count($newsletters) ? $newsletters[$nlid] : NULL;
}

/**
 * Loads multiple newsletters in order.
 */
function sunmailer_newsletter_load_multiple($nlids) {
  $query = "SELECT * FROM {sunmailer_newsletters} WHERE nlid IN (" . db_placeholders($nlids) . ") ORDER BY weight, nlid ASC";
  return _sunmailer_newsletters_load($query, $nlids);
}

/**
 * Loads all newsletters in order.
 */
function sunmailer_newsletter_load_all() {
  $query = "SELECT * FROM {sunmailer_newsletters} ORDER BY weight, nlid ASC";
  return _sunmailer_newsletters_load($query, array());
}

/**
 * Deletes a newsletter as well as associated sections and subscriptions.
 */
function sunmailer_newsletter_delete($nlid) {
  db_query("DELETE su FROM {sunmailer_sections} se, {sunmailer_subscriptions} su WHERE se.sid = su.sid AND se.nlid = %d", $nlid);
  db_query("DELETE FROM {sunmailer_sections} WHERE nlid = %d", $nlid);
  $logo_fid = db_result(db_query("SELECT logo_fid FROM {sunmailer_newsletters} WHERE nlid = %d", $nlid));
  _sunmailer_file_delete($logo_fid);
  db_query("DELETE FROM {sunmailer_newsletters} WHERE nlid = %d", $nlid);
}

/**
 * Loads newsletter settings.
 */
function sunmailer_newsletter_settings_load($nlid) {
  // Any existing newsletter should have an entry in the database.
  $newsletter_settings = db_fetch_object(db_query("SELECT * FROM {sunmailer_newsletter_settings} WHERE nlid = %d", $nlid));
  if ($newsletter_settings) {
    return $newsletter_settings;
  }

  // Create the default settings for a new newsletter.
  $newsletter_settings = new stdClass();
  $newsletter_settings->nlid = $nlid;
  $newsletter_settings->cutoff_date = time();
  $newsletter_settings->last_sent_date = 0;
  $newsletter_settings->message = '';
  return $newsletter_settings;
}

/**
 * Saves newsletter settings.
 */
function sunmailer_newsletter_settings_save($newsletter_settings) {
  $exists = db_result(db_query("SELECT COUNT(*) FROM {sunmailer_newsletter_settings} WHERE nlid = %d", $newsletter_settings->nlid));
  drupal_write_record('newsletter_settings', $newsletter_settings, $exists ? 'nlid' : array());
}

/**
 * Deletes newsletter settings.
 */
function sunmailer_newsletter_settings_delete($nlid) {
  db_query("DELETE FROM {sunmailer_newsletter_settings} WHERE nlid = %d", $nlid);
}

/**
 * Loads a newsletter mailing schedule.
 */
function sunmailer_newsletter_schedule_load($nlid) {
  $newsletter_schedule = db_fetch_object(db_query("SELECT * FROM {sunmailer_newsletter_schedules} WHERE nlid = %d", $nlid));
  if ($newsletter_schedule) {
    return $newsletter_schedule;
  }
  $newsletter_schedule = new stdClass();
  $newsletter_schedule->nlid = $nlid;
  $newsletter_schedule->type = 'none';
  $newsletter_schedule->next_date = 0;
  $newsletter_schedule->unit = '';
  $newsletter_schedule->data = '';
  return $newsletter_schedule;
}

/**
 * Saves a newsletter mailing schedule.
 */
function sunmailer_newsletter_schedule_save($newsletter_schedule) {
  if ($newsletter_schedule->type == 'none') {
    sunmailer_newsletter_schedule_delete($newsletter_schedule->nlid);
  }
  $exists = db_result(db_query("SELECT COUNT(*) FROM {sunmailer_newsletter_schedules} WHERE nlid = %d", $newsletter_schedule->nlid));
  drupal_write_record('sunmailer_newsletter_schedules', $newsletter_schedule, $exists ? 'nlid' : array());
}

/**
 * Deletes a newsletter mailing schedule.
 */
function sunmailer_newsletter_schedule_delete($nlid) {
  db_query("DELETE FROM {sunmailer_newsletter_schedules} WHERE nlid = %d", $nlid);
}

/**
 * Loads one or more sections using a query.
 */
function _sunmailer_sections_load($query, $args = array()) {
  $sections = array();
  $result = db_query($query, $args);
  while ($row = db_fetch_object($result)) {
    $sections[$row->sid] = $row;
  }
  return $sections;
}

/**
 * Loads one section.
 */
function sunmailer_section_load($sid) {
  $query = "SELECT * FROM {sunmailer_sections} WHERE sid = %d";
  $sections = _sunmailer_sections_load($query, $sid);
  return count($sections) ? $sections[$sid] : NULL;
}

/**
 * Loads all sections in a newsletter.
 */
function sunmailer_section_load_newsletter($nlid) {
  $query = "SELECT * FROM {sunmailer_sections} WHERE nlid = %d ORDER BY weight, sid ASC";
  return _sunmailer_sections_load($query, $nlid);
}

/**
 * Loads all sections in order.
 */
function sunmailer_section_load_all() {
  $query = "SELECT * FROM {sunmailer_sections} ORDER BY weight, sid ASC";
  return _sunmailer_sections_load($query);
}

/**
 * Deletes a section as well as its associated subscriptions.
 */
function sunmailer_section_delete($sid) {
  db_query("DELETE FROM {sunmailer_subscriptions} WHERE sid = %d", $sid);
  db_query("DELETE FROM {sunmailer_sections} WHERE sid = %d", $sid);
}

/**
 * Loads one or more designs using a query.
 */
function _sunmailer_designs_load($query, $args = array())
{
  $designs = array();

  // Load basic information for each design.
  $result = db_query($query, $args);
  while ($row = db_fetch_object($result))
  {
    $designs[$row->did] = $row;
  }

  // Load extended information from other database tables.
  foreach ($designs as &$design)
  {
    $design->xsl_path = db_result(db_query("SELECT filepath FROM {files} WHERE fid = %d", $design->fid));
  }

  return $designs;
}

/**
 * Retrieves one design.
 */
function sunmailer_design_load($did) {
  $query = "SELECT * FROM {sunmailer_designs} WHERE did = %d";
  $designs = _sunmailer_designs_load($query, $did);
  return count($designs) ? $designs[$did] : NULL;
}

/**
 * Retrieves all designs.
 */
function sunmailer_design_load_all() {
  return _sunmailer_designs_load("SELECT * FROM {sunmailer_designs}");
}

/**
 * Deletes a design.
 */
function sunmailer_design_delete($did) {
  $fid = db_result(db_query("SELECT fid FROM {sunmailer_designs} WHERE did = %d", $did));
  _sunmailer_file_delete($fid);
  db_query("DELETE FROM {sunmailer_designs} WHERE did = %d", $did);
}

/**
 * Loads the subscription for one subscriber.
 */
function sunmailer_subscription_load($subscriber) {
  $result = db_query("SELECT se.nlid, se.sid FROM {sunmailer_newsletters} n, {sunmailer_sections} se, {sunmailer_subscriptions} su WHERE n.nlid = se.nlid AND se.sid = su.sid AND su.smid = %d ORDER BY n.weight, n.nlid, se.weight, se.sid ASC", $subscriber->smid);
  $subscription = array();
  while ($row = db_fetch_object($result)) {
    $subscription[$row->nlid][] = $row->sid;
  }
  return $subscription;
}

/**
 * Updates the subscription for a subscriber.
 */
function sunmailer_subscription_save($subscriber) {
  db_query("DELETE FROM {sunmailer_subscriptions} WHERE smid = %d", $subscriber->smid);
  $placeholders = array();
  $args = array();
  foreach ($subscriber->subscription as $newsletter_subscription) {
    foreach ($newsletter_subscription as $sid) {
      $placeholders[] = "(%d, %d)";
      $args[] = $subscriber->smid;
      $args[] = $sid;
    }
  }
  if (count($args))
  {
    db_query("INSERT INTO {sunmailer_subscriptions} VALUES " . implode(', ', $placeholders), $args);
  }
}

/**
 * Loads a subscriber.
 */
function sunmailer_subscriber_load($mail) {
  // Try loading the subscriber from the database.
  $subscriber = db_fetch_object(db_query("SELECT smid, mail FROM {sunmailer_subscribers} WHERE mail = '%s'", $mail));
  if ($subscriber) {
    $subscriber->subscription = sunmailer_subscription_load($subscriber);
    return $subscriber;
  }

  // Build a new subscriber otherwise.
  $subscriber = new stdClass();
  $subscriber->smid = NULL;
  $subscriber->mail = $mail;
  $subscriber->subscription = array();
  return $subscriber;
}

/**
 * Saves a subscriber.
 */
function sunmailer_subscriber_save($subscriber) {
  if (count($subscriber->subscription)) {
    if (!$subscriber->smid) {
      drupal_write_record('sunmailer_subscribers', $subscriber);
    }
    sunmailer_subscription_save($subscriber);
  }
  else {
    sunmailer_subscriber_delete($subscriber);
  }
}

/**
 * Deletes a subscriber.
 */
function sunmailer_subscriber_delete($subscriber) {
  if (!$subscriber->smid) {
    return;
  }
  db_query("DELETE FROM {sunmailer_subscriptions} WHERE smid = %d", $subscriber->smid);
  db_query("DELETE FROM {sunmailer_subscribers} WHERE smid = %d", $subscriber->smid);
}

/**
 * Loads all subscriptions for one newsletter.
 */
function sunmailer_newsletter_subscriptions_load($newsletter) {
  $result = db_query("SELECT sm.mail, su.sid FROM {sunmailer_sections} se, {sunmailer_subscriptions} su, {sunmailer_subscribers} sm WHERE se.sid = su.sid AND su.smid = sm.smid AND se.nlid = %d ORDER BY su.smid, se.weight, se.sid ASC", $newsletter->nlid);
  $subscriptions = array();
  while ($row = db_fetch_object($result)) {
    $subscriptions[$row->mail][] = $row->sid;
  }
  return $subscriptions;
}

/**
 * Loads all subscriptions for one newsletter, grouping common subscriptions.
 */
function sunmailer_newsletter_grouped_subscriptions_load($newsletter) {
  $subscriptions = sunmailer_newsletter_subscriptions_load($newsletter);
  $grouped_subscriptions = array();
  foreach ($subscriptions as $mail => $sids) {
    $grouped_subscriptions[implode(',', $sids)][] = $mail;
  }
  return $grouped_subscriptions;
}

/**
 * Generates a link to access the subscription page.
 */
function sunmailer_subscription_link($mail = NULL, $nlids = array(), $sids = array())
{
  // Check for an anonymous link.
  if (!$mail) {
    return url('sunmailer/subscribe', array('absolute' => TRUE));
  }

  // Embed the mail and hash into the link.
  $query = array(
    'mail' => $mail,
    'hash' => sunmailer_subscriber_hash($mail),
  );

  // Optionally embed a subscription into the link.
  if (count($nlids)) {
    $query['nlids'] = implode(',', $nlids);
    if (count($sids)) {
      $query['sids'] = implode(',', $sids);
    }
  }
  return url('sunmailer/subscribe', array('absolute' => TRUE, 'query' => $query));
}

/**
 * Returns a subscriber's unique hash used for authentication.
 */
function sunmailer_subscriber_hash($mail)
{
  return md5($mail . drupal_get_private_key());
}

/**
 * Sets up an instance of the PHPMailer class.
 * It still needs the email's contents and recipients.
 */
function sunmailer_get_mailer() {
  $path = variable_get('sunmailer_phpmailer_path', '');
  require_once $path ? "$path/class.phpmailer.php" : 'class.phpmailer.php';
  $mailer = new PHPMailer();
  $mailer->FromName = variable_get('sunmailer_from_name', '');
  $mailer->From = variable_get('sunmailer_from_email', '');
  $mailer->Host = variable_get('sunmailer_host', 'localhost');
  $mailer->Port = variable_get('sunmailer_port', 25);
  switch (variable_get('sunmailer_method', 'mail')) {
  case 'mail':
    $mailer->IsMail();
    break;
  case 'sendmail':
    $mailer->IsSendmail();
    $mailer->Sendmail = variable_get('sunmailer_sendmail_path', '/usr/bin/sendmail');
    break;
  case 'smtp':
    $mailer->IsSMTP();
    $mailer->SMTPAuth = variable_get('sunmailer_smtp_auth', FALSE);
    if ($mailer->SMTPAuth) {
      $mailer->Username = variable_get('sunmailer_smtp_user', '');
      $mailer->Password = variable_get('sunmailer_smtp_pass', '');
    }
    $mailer->SMTPSecure = variable_get('sunmailer_smtp_secure', '');
    break;
  }
  $mailer->CharSet = variable_get('sunmailer_char_set', 'utf-8');
  $mailer->WordWrap = variable_get('sunmailer_word_wrap', 80);
  return $mailer;
}

/**
 * Splits a list of recipients into groups for mailing.
 */
function sunmailer_split_addresses($addresses) {
  $group_size = variable_get('sunmailer_max_recipients', 1);
  $group_size = $group_size ? $group_size : count($addresses);
  $address_groups = array();
  for ($i = 0; $i < count($addresses); $i += $group_size) {
    $address_groups[] = array_slice($addresses, $i, $group_size);
  }
  return $address_groups;
}

/**
 * Sends an email.
 */
function sunmailer_send_mail($mailer, $addresses, $subject, $body, $is_html = FALSE, $alt_body = NULL) {
  // Set contents of email.
  $mailer->Subject = $subject;
  $mailer->Body = $body;
  $mailer->IsHTML($is_html);
  $mailer->AltBody = $is_html ? $alt_body : NULL;
  // Use To: for individual emails, Bcc: for group emails.
  $address_groups = sunmailer_split_addresses($addresses);
  foreach ($address_groups as $addresses) {
    $mailer->ClearAllRecipients();
    if (count($addresses) == 1) {
      $mailer->AddAddress(array_pop($addresses));
    }
    else {
      foreach ($addresses as $address) {
        $mailer->AddBCC($address);
      }
    }
    // Send mail.
    $retries = variable_get('sunmailer_retries', 2);
    while ($retries >= 0) {
      if ($mailer->Send()) {
        break;
      }
      $retries--;
      if ($retries >= 0) {
        watchdog('mail', 'Email failed to send (@error), retrying...', array('@error' => $mailer->ErrorInfo), WATCHDOG_WARNING);
      }
      else {
        watchdog('mail', 'Email failed to send (@error), giving up', array('@error' => $mailer->ErrorInfo), WATCHDOG_ERROR);
      }
    }
  }
}
